/*
 * Author: Andreas Ericsson <ae@op5.se>
 *
 * Copyright(C) 2005 OP5 AB
 * All rights reserved.
 *
 */

#include <stdarg.h>
#include <limits.h>
#include <regex.h>
#include "common.h"
#include "utils.h"
#include "runcmd.h"

#define NP_SNMP_REGEX 1
#define NP_SNMP_INTEGER 2
#define PROG_VERSION "1.2.0"

char progname[PATH_MAX];
static char *host;

typedef struct np_range {
	int lo_warn;
	int hi_warn;
	int lo_crit;
	int hi_crit;
	int flags; /* 0 when unspec, b1 when lo_warn, b2 when hi_warn
				* b3 when lo_crit, b4 when hi_crit */
} np_range;

struct snmp_check {
	const char *name;
	const char *oid;
	np_range range;
	const char *regex;
};

static int debug = 0;

#define NP_RANGE_LO_WARN 1
#define NP_RANGE_HI_WARN 2
#define NP_RANGE_LO_CRIT 4
#define NP_RANGE_HI_CRIT 8

struct snmp_check *oid_list;
struct snmp_check oid_list_all[] = {
	/* { "Fan #1", ".1.3.6.1.4.1.9303.4.1.2.3.1.2.1", { 10, 0, 0, 0, NP_RANGE_LO_WARN }, NULL }, */
	/* { "Fan #2", ".1.3.6.1.4.1.9303.4.1.2.3.1.2.2", { 10, 0, 0, 0, NP_RANGE_LO_WARN }, NULL }, */
	/* { "Fan #3", ".1.3.6.1.4.1.9303.4.1.2.3.1.2.3", { 10, 0, 0, 0, NP_RANGE_LO_WARN }, NULL }, */
	/* { "Fan #4", ".1.3.6.1.4.1.9303.4.1.2.3.1.2.4", { 10, 0, 0, 0, NP_RANGE_LO_WARN }, NULL }, */
/*
check_pfvoltstatus_one SFP Voltage Status #1 .1.3.6.1.4.1.9303.4.1.4.1.13.201    0|3 = OK
check_pfvoltstatus_one SFP Voltage Status #2 .1.3.6.1.4.1.9303.4.1.4.1.13.202    0|3 = OK
*/
/*	{ "SFP Voltage Status #1", ".1.3.6.1.4.1.9303.4.1.4.1.13.201", { 0, 0, 0, 0, 0 }, "0\\|3" }, */ //Peter  ...25? =3
/*	{ "SFP Voltage Status #2", ".1.3.6.1.4.1.9303.4.1.4.1.13.202", { 0, 0, 0, 0, 0 }, "0\\|3" }, */ //Peter  ...26? =0
/*
check_pfsfprxpower_one SFP RX Power #1 .1.3.6.1.4.1.9303.4.1.4.1.20.201 150:700 = OK
check_pfsfprxpower_two SFP RX Power #2 .1.3.6.1.4.1.9303.4.1.4.1.20.202 150:700 = OK
*/
	/* { "SFP RX Power #1", ".1.3.6.1.4.1.9303.4.1.4.1.19.201", { 150, 700, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */
	/* { "SFP RX Power #2", ".1.3.6.1.4.1.9303.4.1.4.1.19.202", { 150, 700, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */
/*	{ "SFP RX Power #3", ".1.3.6.1.4.1.9303.4.1.4.1.20.201", { 150, 700, 0, 0, 3 }, NULL }, */ //Peter ..25? = 55
/*	{ "SFP RX Power #4", ".1.3.6.1.4.1.9303.4.1.4.1.20.202", { 150, 700, 0, 0, 3 }, NULL }, */ //Peter ..26? = 0
//	{ "SFP RX Power #3", ".1.3.6.1.4.1.9303.4.1.4.1.20.25", { 150, 700, 0, 0, 3 }, NULL },  //Peter ..25? = 55
//	{ "SFP RX Power #4", ".1.3.6.1.4.1.9303.4.1.4.1.20.26", { 150, 700, 0, 0, 3 }, NULL },  //Peter ..26? = 0
/*
check_pfsfptempstatus_one  Not used .1.3.6.1.4.1.9303.4.1.4.1.11.201 0|3 = OK
check_pfsfptempstatus_two  Not used .1.3.6.1.4.1.9303.4.1.4.1.11.202 0|3 = OK
*/
/*	 { "SFP Tempstatus #1", ".1.3.6.1.4.1.9303.4.1.4.1.11.201", { 0, 0, 0, 0, 0 }, "0\\|3" }, */
/*	 { "SFP Tempstatus #2", ".1.3.6.1.4.1.9303.4.1.4.1.11.202", { 0, 0, 0, 0, 0 }, "0\\|3" }, */
/*
check_pfsfptemp_one SFP Temperature #1 .1.3.6.1.4.1.9303.4.1.4.1.12.201 10:60 = OK
check_pfsfptemp_two SFP Temperature #2 .1.3.6.1.4.1.9303.4.1.4.1.12.202 10:60 = OK
*/
/*	{ "SFP Temp #1", ".1.3.6.1.4.1.9303.4.1.4.1.12.201", { 10, 60, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */ //Peter
/*	{ "SFP Temp #2", ".1.3.6.1.4.1.9303.4.1.4.1.12.202", { 10, 60, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */ //Peter
//	{ "SFP Temp #1", ".1.3.6.1.4.1.9303.4.1.4.1.12.25", { 10, 60, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, //Peter
//	{ "SFP Temp #2", ".1.3.6.1.4.1.9303.4.1.4.1.12.26", { 10, 60, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, //Peter
/*                    
check_pfsfptxpower_one SFP TX Power #1 .1.3.6.1.4.1.9303.4.1.4.1.18.201 150:400 = OK
check_pfsfptxpower_two SFP TX Power #2 .1.3.6.1.4.1.9303.4.1.4.1.18.202 150:400 = OK
*/
	/* { "SFP TX Power #1", ".1.3.6.1.4.1.9303.4.1.4.1.17.201", { 150, 400, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */
	/* { "SFP TX Power #2", ".1.3.6.1.4.1.9303.4.1.4.1.17.202", { 150, 400, 0, 0, NP_RANGE_LO_WARN | NP_RANGE_HI_WARN }, NULL }, */
/*	{ "SFP TX Power #3", ".1.3.6.1.4.1.9303.4.1.4.1.18.201", { 150, 400, 0, 0, 3 }, NULL }, */ //Peter ..25 = 265
/*	{ "SFP TX Power #4", ".1.3.6.1.4.1.9303.4.1.4.1.18.202", { 150, 400, 0, 0, 3 }, NULL }, */ //Peter ..26 = 0
/*
check_pftempmeasured_zero  Temperature #0 .1.3.6.1.4.1.9303.4.1.2.1.1.2.0 1000:6000 = OK (celcius/100)
check_pftempmeasured_one   Temperature #1 .1.3.6.1.4.1.9303.4.1.2.1.1.2.1 1000:6000 = OK (celcius/100)
check_pftempmeasured_two   Temperature #2 .1.3.6.1.4.1.9303.4.1.2.1.1.2.2 1000:6000 = OK (celcius/100)
check_pftempmeasured_three Temperature #3 .1.3.6.1.4.1.9303.4.1.2.1.1.2.3 1000:6000 = OK (celcius/100)
*/
///	{ "Temperature #0", ".1.3.6.1.4.1.9303.4.1.2.1.1.2.0", { 1000, 6000, 0, 0, 3 }, NULL },
///	{ "Temperature #1", ".1.3.6.1.4.1.9303.4.1.2.1.1.2.1", { 1000, 6000, 0, 0, 3 }, NULL },
///	{ "Temperature #2", ".1.3.6.1.4.1.9303.4.1.2.1.1.2.2", { 1000, 6000, 0, 0, 3 }, NULL },
///	{ "Temperature #3", ".1.3.6.1.4.1.9303.4.1.2.1.1.2.3", { 1000, 6000, 0, 0, 3 }, NULL },
/*
check_pftempstatus_zero  Not used .1.3.6.1.4.1.9303.4.1.2.1.1.7.0   0 = OK
check_pftempstatus_one   Not used .1.3.6.1.4.1.9303.4.1.2.1.1.7.1   0 = OK
check_pftempstatus_two   Not used .1.3.6.1.4.1.9303.4.1.2.1.1.7.2   0 = OK
check_pftempstatus_three Not used .1.3.6.1.4.1.9303.4.1.2.1.1.7.3   0 = OK
*/
	
	/* Borde kolla vilka aktiv? */
	{ "SFP RX Power Status #1", ".1.3.6.1.4.1.9303.4.1.4.1.19.25", { 0, 0, 0, 0, 0 }, "0\\|3" },
	{ "SFP RX Power Status #2", ".1.3.6.1.4.1.9303.4.1.4.1.19.26", { 0, 0, 0, 0, 0 }, "0\\|3" },

	{ "SFP TX Power Status #1", ".1.3.6.1.4.1.9303.4.1.4.1.17.25", { 0, 0, 0, 0, 0 }, "0\\|3" },
	{ "SFP TX Power Status #2", ".1.3.6.1.4.1.9303.4.1.4.1.17.26", { 0, 0, 0, 0, 0 }, "0\\|3" },

	{ "SFP TX Current Status #1", ".1.3.6.1.4.1.9303.4.1.4.1.15.25", { 0, 0, 0, 0, 0 }, "0\\|3" },
	{ "SFP TX Current Status #2", ".1.3.6.1.4.1.9303.4.1.4.1.15.26", { 0, 0, 0, 0, 0 }, "0\\|3" },

	{ "SFP Voltage Status #1", ".1.3.6.1.4.1.9303.4.1.4.1.13.25", { 0, 0, 0, 0, 0 }, "0\\|3" },
	{ "SFP Voltage Status #2", ".1.3.6.1.4.1.9303.4.1.4.1.13.26", { 0, 0, 0, 0, 0 }, "0\\|3" },

	{ "SFP Tempstatus #1", ".1.3.6.1.4.1.9303.4.1.4.1.11.25", { 0, 0, 0, 0, 0 }, "0\\|3" }, 
	{ "SFP Tempstatus #2", ".1.3.6.1.4.1.9303.4.1.4.1.11.26", { 0, 0, 0, 0, 0 }, "0\\|3" }, 
/* Peter....  */
	{ "Tempstatus #0", ".1.3.6.1.4.1.9303.4.1.2.1.1.7.0", { 0, 0, 0, 0, 0 }, "0\\|4" },
	{ "Tempstatus #1", ".1.3.6.1.4.1.9303.4.1.2.1.1.7.1", { 0, 0, 0, 0, 0 }, "0\\|4" },
	{ "Tempstatus #2", ".1.3.6.1.4.1.9303.4.1.2.1.1.7.2", { 0, 0, 0, 0, 0 }, "0\\|4" },
	{ "Tempstatus #3", ".1.3.6.1.4.1.9303.4.1.2.1.1.7.3", { 0, 0, 0, 0, 0 }, "0\\|4" },

/*
check_pfvoltentry_zero  Voltage #0 .1.3.6.1.4.1.9303.4.1.2.2.1.6.0  0|3 = OK
check_pfvoltentry_zero  Voltage #1 .1.3.6.1.4.1.9303.4.1.2.2.1.6.1  0|3 = OK
check_pfvoltentry_zero  Voltage #2 .1.3.6.1.4.1.9303.4.1.2.2.1.6.2  0|3 = OK
check_pfvoltentry_zero  Voltage #3 .1.3.6.1.4.1.9303.4.1.2.2.1.6.3  0|3 = OK
check_pfvoltentry_zero  Voltage #4 .1.3.6.1.4.1.9303.4.1.2.2.1.6.4  0|3 = OK
check_pfvoltentry_zero  Voltage #5 .1.3.6.1.4.1.9303.4.1.2.2.1.6.5  0|3 = OK
check_pfvoltentry_zero  Voltage #6 .1.3.6.1.4.1.9303.4.1.2.2.1.6.6  0|3 = OK
check_pfvoltentry_zero  Voltage #7 .1.3.6.1.4.1.9303.4.1.2.2.1.6.7  0|3 = OK
*/
	/* Peter....  */
//	{ "Voltage #0", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.0", { 0, 0, 0, 0, 0 }, NULL },
//	{ "Voltage #1", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.1", { 0, 0, 0, 0, 0 }, NULL },
//	{ "Voltage #2", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.0", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #3", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.1", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #4", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.4", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #5", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.5", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #6", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.6", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #7", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.7", { 0, 0, 0, 0, 0 }, "0\\|3" },
//	{ "Voltage #8", ".1.3.6.1.4.1.9303.4.1.2.2.1.6.8", { 0, 0, 0, 0, 0 }, "0\\|3" },
   	{ "Power supply A", ".1.3.6.1.4.1.9303.4.1.2.2.1.3.0", { 11000, 0, 11000, 0, NP_RANGE_LO_CRIT }, NULL },
   	{ "Power supply B", ".1.3.6.1.4.1.9303.4.1.2.2.1.3.1", { 11000, 0, 11000, 0, NP_RANGE_LO_CRIT }, NULL },
	{ NULL, NULL, { 0, 0, 0, 0, 0 }, NULL }
};


struct snmp_check oid_list_power[] = {
   	{ "Power supply A", ".1.3.6.1.4.1.9303.4.1.2.2.1.3.0", { 11000, 0, 11000, 0, NP_RANGE_LO_CRIT }, NULL },
   	{ "Power supply B", ".1.3.6.1.4.1.9303.4.1.2.2.1.3.1", { 11000, 0, 11000, 0, NP_RANGE_LO_CRIT }, NULL },
	{ NULL, NULL, { 0, 0, 0, 0, 0 }, NULL }
};


void print_help()
{
	printf("Usage: %s -H host [ -C community ] [ -p port ]\n\n", progname);
	puts("  -H, --host       Host to check");
	puts("  -C, --community  Community-string to use when connecting (default: public)");
	puts("  -p, --port       Port to use (default: 161)");
        puts("  -P, --power      Only check power status (on _both_ power supplies) ");
	puts("  -h, --help       This cruft");
	puts("  -V, --version    Version info");

	putchar('\n');
}

static int num_oids = 0;

static int np_range_match(int val, np_range *range)
{
	if(debug)
		printf("matching %d against range { %d, %d, %d, %d, %d }\n",
			   val, range->lo_warn, range->hi_warn,
			   range->lo_crit, range->hi_crit, range->flags);

	if(!range) return STATE_OK;

//	if(range->flags & NP_RANGE_LO_CRIT && val < range->lo_crit ||
//	   range->flags & NP_RANGE_HI_CRIT && val > range->hi_crit)
	if((range->flags & NP_RANGE_LO_CRIT && val < range->lo_crit) ||
	   (range->flags & NP_RANGE_HI_CRIT && val > range->hi_crit))
	{
		return STATE_CRITICAL;
	}

//	if(range->flags & NP_RANGE_LO_WARN && val < range->lo_warn ||
//	   range->flags & NP_RANGE_HI_WARN && val > range->hi_warn)
	if((range->flags & NP_RANGE_LO_WARN && val < range->lo_warn) ||
	   (range->flags & NP_RANGE_HI_WARN && val > range->hi_warn))
	{
		return STATE_WARNING;
	}

	return STATE_OK;
}

#if 0
static int np_range_match(int val, np_range *range)
{
	if(debug)
		printf("matching %d against range { %d, %d, %d, %d, %d }\n",
			   val, range->lo_warn, range->hi_warn,
			   range->lo_crit, range->hi_crit, range->flags);

	if(range->flags & NP_RANGE_LO_CRIT && val >= range->lo_crit) {
		if(range->flags & NP_RANGE_HI_CRIT) {
			/* high and low critical boundaries */
			if(val <= range->hi_crit)
				return STATE_CRITICAL;
		}
		/* low critical boundary but no upper one. val is above lo_crit */
		else return STATE_CRITICAL;
	}

	/* only remaining condition left to return critical;
	 * high critical is specified, low critical is not. value is lower than
	 * high critical */
	if(range->flags & NP_RANGE_HI_CRIT && val <= range->hi_crit)
		return STATE_CRITICAL;

	if(range->flags & NP_RANGE_LO_WARN && val >= range->lo_warn) {
		if(range->flags & NP_RANGE_LO_WARN) {
			if(val <= range->hi_warn)
				return STATE_WARNING;
		}
		else return STATE_WARNING;
	}

	return STATE_OK;
}
#endif

static int report_status(int *list)
{
	int status = STATE_OK;
	int *warn_list = NULL, *crit_list = NULL;
	int crit = 0, warn = 0;
	int i;

	for(i = 0; i < num_oids; i++) {
		if(list[i] == STATE_CRITICAL) {
			status = STATE_CRITICAL;
			if(!crit_list) crit_list = malloc(sizeof(int) * (num_oids - i));
			crit_list[crit++] = i;
		}
		else if(list[i] == STATE_WARNING) {
			if(status != STATE_CRITICAL) status = STATE_WARNING;
			if(!warn_list) warn_list = malloc(sizeof(int) * (num_oids - i));
			warn_list[warn++] = i;
		}
	}
	if(!warn && !crit) {
		printf("All checks are OK!\n");
		return status;
	}

	if(crit) {
		printf("CRITICAL (%i) - ", crit);
		for(i = 0; i < crit; i++) {
			printf("%s", oid_list[crit_list[i]].name);
			if(i < crit - 1) printf(", ");
			else if(warn) printf(" ::  ");
		}
	}
	if(warn) {
		printf("WARNING (%i) - ", warn);
		for(i = 0; i < warn; i++) {
			printf("%s", oid_list[warn_list[i]].name);
			if(i < warn - 1) printf(", ");
		}
	}

	putchar('\n');

	return status;
}

static int check_power_channel(char *cmd);
static int check_power_channel(char *cmd)
{
   struct output chld_out, chld_err;
   int result, line;
//   unsigned long val;

   result = np_runcmd(cmd , &chld_out, &chld_err, 0);
   if(debug) printf("np_runcmd returned %d\n", result);
   if(result != 0) {
      if(!chld_out.buflen) {
	 printf("CRITICAL - Error while fetching data from '%s'", host);
	 if(chld_err.buflen) printf(": %s\n", chld_err.buf);
	 exit(STATE_CRITICAL);
      }
   }
   /* Disabling powersupplies that are notPresent(3) or disabled(4) */
   /* This should only be one when disabled i guess                 */
   for(line = 0; line < chld_out.lines; line++) {
      if(!strcmp(chld_out.line[line], "3") || !strcmp(chld_out.line[line], "4"))
	{
	   oid_list[num_oids -2 + line ].name = NULL; 
	   oid_list[num_oids -2 + line ].oid = NULL; 
	}
   }
	return 0;
}


static int check_snmp_data(char *cmd);
static int check_snmp_data(char *cmd)
{
	struct output chld_out, chld_err;
	int result, i, line, foo;
	unsigned long val;
	regex_t re;
	int *status_list;

	status_list = malloc(sizeof(int) * num_oids);
	if(!status_list) {
		printf("malloc(3) failed: %s\n", strerror(errno));
		exit(STATE_UNKNOWN);
	}
	for(i = 0; i < num_oids; i++) status_list[i] = STATE_OK;

	result = np_runcmd(cmd, &chld_out, &chld_err, 0);
	if(debug) printf("np_runcmd returned %d\n", result);
	if(result != 0) {
		if(!chld_out.buflen) {
			printf("CRITICAL - Error while fetching data from '%s'", host);
			if(chld_err.buflen) printf(": %s\n", chld_err.buf);
			exit(STATE_CRITICAL);
		}
	}

	if(debug > 2) {
		puts("stdout output:\n--------------");
		for(line = 0; line < chld_out.lines; line++) {
			printf("%d: %s\n", line, chld_out.line[line]);
		}
		puts("stderr output:\n--------------");
		for(line = 0; line < chld_err.lines; line++) {
			printf("%d: %s\n", line, chld_err.line[line]);
		}
	}

	for(i = 0; oid_list[i].name != NULL; i++) {
		if(i > chld_out.lines) {
			printf("CRITICAL - Failed to receive all data\n");
			exit(STATE_CRITICAL);
		}

		
		if(debug)
			printf("\n*** %s :: %s\nt", oid_list[i].name, oid_list[i].oid);

		if(oid_list[i].regex) {
			foo = regcomp(&re, oid_list[i].regex, REG_EXTENDED | REG_NOSUB);
			if(debug) {
				if(foo) printf("regcomp(3) returned %d\n", foo);

				printf("matching regex '%s' against string '%s'\n",
					   oid_list[i].regex, chld_out.line[i]);
			}

			if((regexec(&re, chld_out.line[i], 0, NULL, 0)) != REG_NOMATCH) {
				if(debug) printf("No match (setting to critical)\n");
// peter				status_list[i] == STATE_CRITICAL;
				status_list[i] = STATE_CRITICAL;
			}
			else if(debug) puts("OK");
		}
		else {
			val = strtoul(chld_out.line[i], NULL, 10);
			status_list[i] = np_range_match(val, &oid_list[i].range);
			if(debug)
				printf("result was %s\n", state_text(status_list[i]));
		}
	}

	return report_status(status_list);
}

int main(int argc, char **argv)
{
	char *community = "public", *cmd = NULL, *port = "161";
   char *cmd_tmp;
   int dualpower = 0, onlypower = 0;
//   struct snmp_check *oid_list_in_use;

	char *oid_str = NULL, *p;
	int i, foo, len = 0, cmd_len;
	const struct option longopts[] = {
		{ "host", required_argument, 0, 'H' },
		{ "community", required_argument, 0, 'C' },
		{ "port", required_argument, 0, 'p' },
		{ "help", no_argument, 0, 'h' },
		{ "version", no_argument, 0, 'V' },
		{ "dualpower", no_argument, 0, 'd' },
		{ "power", no_argument, 0, 'P' },
		{ "verbose", no_argument, 0, 'v' }
	};

	char *base = strrchr(argv[0], '/');
	strncpy(progname, (base ? base + 1 : argv[0]), PATH_MAX);
	while((i = getopt_long(argc, argv, "H:C:p:VhvdP", longopts, &foo)) != EOF) {
		switch(i) {
		case 'H':
			host = optarg;
			break;
		case 'C':
			community = optarg;
			break;
		case 'p':
			port = optarg;
			break;
		case 'v':
			debug++;
			break;
		 case 'd':
			dualpower++;
			break;
		 case 'P':
			onlypower++;
			break;
		case 'h':
		default:
			print_help();
			/* fallthrough */
		case 'V':
			printf("%s v.%s \n", progname, PROG_VERSION);
			return STATE_UNKNOWN;
		}
	}

	/* make sure we have what we need */
	if(!host || !host[0] || !community[0] || !port[0]) {
		print_help();
		return STATE_UNKNOWN;
	}

   if(onlypower==0)
     oid_list = oid_list_all;
   else
     oid_list = oid_list_power;
	len = 1;
	for(i = 0; oid_list[i].oid != NULL; i++) {
		num_oids++;
		len += strlen(oid_list[i].oid) + 1;
		if(debug > 3) {
			printf("oid_list[%d].{name, oid} = {%s, %s}\n", i,
				   oid_list[i].name, oid_list[i].oid);
		}
	}
	
	p = oid_str = malloc(len);
	memset(oid_str, 0, len + 1);
	oid_str[len - 1] = '\0';
	for(i = 0; oid_list[i].oid != NULL; i++) {
		strcat(p, oid_list[i].oid);
		p += strlen(oid_list[i].oid);
		*p++ = ' ';
	}

	/* allocate the command string, fill it with spaces and null-terminate */
#define MIB_COM_OPTS "-Ovq -m: -v 2c -c"
	cmd_len = 5 + strlen(MIB_COM_OPTS) + len + strlen(PATH_TO_SNMPGET) +
		strlen(host) + strlen(community) + strlen(port);
	cmd = malloc(cmd_len);
	memset(cmd, 0, cmd_len);

	p = cmd;
	strcat(p, PATH_TO_SNMPGET);
	p += strlen(PATH_TO_SNMPGET);
	*p++ = ' ';
	strcat(p, host);
	p += strlen(host);
	*p++ = ':';
	strcat(p, port);
	p += strlen(port);
	*p++ = ' ';
	strcat(p, MIB_COM_OPTS);
	p += strlen(MIB_COM_OPTS);
	*p++ = ' ';
	strcat(p, community);
	p += strlen(community);
	*p++ = ' ';
	if(debug > 2) printf("cmd: %s\n", cmd);
   /* Disablig powersupply check for nonpresent/disabled power supplies */
   /* Override with -d (Always check all...)                            */
   cmd_tmp = malloc(strlen(cmd) + 60);
   cmd_tmp = strcat(cmd_tmp,cmd);
   cmd_tmp = strcat(cmd_tmp,".1.3.6.1.4.1.9303.4.1.2.2.1.6.0 .1.3.6.1.4.1.9303.4.1.2.2.1.6.1");
   if(dualpower==0 && onlypower==0) {
      if(debug) printf("**** WARNING!\nDisabling non present power supplies\n");
      check_power_channel(cmd_tmp);
   }
   
	memcpy(p, oid_str, len - 1);

	if(debug) {
		printf("cmd (%d): %s\n", cmd_len, cmd);
		if(debug > 1) {
			printf("cmd_len: %d, strlen(cmd): %d\n", cmd_len, strlen(cmd));
			printf("oid_str(%d): %s\n", len, oid_str);
		}
	}

	return check_snmp_data(cmd);
}
